@@ -3,7 +3,13 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 package org.mozilla.jss;
 
+import java.lang.NullPointerException;
+import java.security.Provider;
+
+import java.io.InputStream;
+
 public final class JSSProvider extends java.security.Provider {
+    public static boolean ENABLE_JSSENGINE = true;
 
     private static final long serialVersionUID = 1L;
     /********************************************************************/
@@ -18,10 +24,72 @@ public final class JSSProvider extends java.security.Provider {
                                            (JSS_MINOR_VERSION * 100 +
                                             JSS_PATCH_VERSION)/10000.0;
 
+    private static JSSLoader loader = new JSSLoader();
+
+    private static CryptoManager cm;
+
     public JSSProvider() {
+        this(CryptoManager.isInitialized());
+    }
+
+    public JSSProvider(boolean initialize) {
         super("Mozilla-JSS", JSS_VERSION,
                 "Provides Signature, Message Digesting, and RNG");
 
+        if (initialize) {
+            initializeProvider();
+        }
+    }
+
+    public JSSProvider(String config_path) throws Exception {
+        this(false);
+
+        configure(config_path);
+    }
+
+    public JSSProvider(InputStream config) throws Exception {
+        this(false);
+
+        cm = loader.init(config);
+        initializeProvider();
+    }
+
+    /**
+     * Configure this instance of JSSProvider with the specified path
+     * to a JSS configuration properties file.
+     *
+     * See JSSLoader's class description for a description of the JSS
+     * configuration properties file and known values.
+     *
+     * If the JSSProvider is already loaded, this is a no-op.
+     */
+    public Provider configure(String arg) {
+        try {
+            cm = loader.init(arg);
+        } catch (NullPointerException npe) {
+            throw npe;
+        } catch (Exception e) {
+            throw new RuntimeException(e.getMessage(), e);
+        }
+
+        initializeProvider();
+
+        return this;
+    }
+
+    /**
+     * Return the CryptoManager this instance was initialized with.
+     */
+    public CryptoManager getCryptoManager() {
+        if (cm == null) {
+            try {
+                cm = CryptoManager.getInstance();
+            } catch (NotInitializedException nie) {}
+        }
+        return cm;
+    }
+
+    protected void initializeProvider() {
         /////////////////////////////////////////////////////////////
         // Signature
         /////////////////////////////////////////////////////////////
@@ -53,6 +121,28 @@ public final class JSSProvider extends java.security.Provider {
         put("Alg.Alias.Signature.SHA256/RSA", "SHA-256/RSA");
         put("Alg.Alias.Signature.SHA256withRSA", "SHA-256/RSA");
 
+        put("Signature.RSASSA-PSS",
+            "org.mozilla.jss.provider.java.security.JSSSignatureSpi$RSAPSSSignature");
+
+        put("Alg.Alias.Signature.1.2.840.113549.1.1.10",     "RSASSA-PSS");
+        put("Alg.Alias.Signature.OID.1.2.840.113549.1.1.10", "RSASSA-PSS");
+
+        put("Signature.SHA-256/RSA/PSS",
+            "org.mozilla.jss.provider.java.security.JSSSignatureSpi$SHA256RSAPSS");
+
+        put("Alg.Alias.Signature.SHA256withRSA/PSS","SHA-256/RSA/PSS");
+
+        put("Signature.SHA-384/RSA/PSS",
+            "org.mozilla.jss.provider.java.security.JSSSignatureSpi$SHA384RSAPSS");
+
+        put("Alg.Alias.Signature.SHA384withRSA/PSS","SHA-384/RSA/PSS");
+
+        put("Signature.SHA-512/RSA/PSS",
+            "org.mozilla.jss.provider.java.security.JSSSignatureSpi$SHA512RSAPSS");
+
+        put("Alg.Alias.Signature.SHA512withRSA/PSS","SHA-512/RSA/PSS");
+
+
         put("Signature.SHA-384/RSA",
             "org.mozilla.jss.provider.java.security.JSSSignatureSpi$SHA384RSA");
         put("Alg.Alias.Signature.SHA384/RSA", "SHA-384/RSA");
@@ -156,6 +246,9 @@ public final class JSSProvider extends java.security.Provider {
         put("AlgorithmParameters.RC2AlgorithmParameters",
             "org.mozilla.jss.provider.java.security.RC2AlgorithmParameters");
 
+        put("AlgorithmParameters.RSAPSSAlgorithmParameters",
+            "org.mozilla.jss.provider.java.security.RSAPSSAlgorithmParameters");
+
         /////////////////////////////////////////////////////////////
         // Cipher
         /////////////////////////////////////////////////////////////
@@ -321,10 +414,22 @@ public final class JSSProvider extends java.security.Provider {
         put("Alg.Alias.TrustManagerFactory.PKIX", "NssX509");
         put("Alg.Alias.TrustManagerFactory.X509", "NssX509");
         put("Alg.Alias.TrustManagerFactory.X.509", "NssX509");
+
+        /////////////////////////////////////////////////////////////
+        // TLS
+        /////////////////////////////////////////////////////////////
+        if (ENABLE_JSSENGINE) {
+            put("SSLContext.Default", "org.mozilla.jss.provider.javax.net.JSSContextSpi");
+            put("SSLContext.SSL", "org.mozilla.jss.provider.javax.net.JSSContextSpi");
+            put("SSLContext.TLS", "org.mozilla.jss.provider.javax.net.JSSContextSpi");
+            put("SSLContext.TLSv1.1", "org.mozilla.jss.provider.javax.net.JSSContextSpi$TLSv11");
+            put("SSLContext.TLSv1.2", "org.mozilla.jss.provider.javax.net.JSSContextSpi$TLSv12");
+            put("SSLContext.TLSv1.3", "org.mozilla.jss.provider.javax.net.JSSContextSpi$TLSv13");
+        }
     }
 
     public String toString() {
-        String mozillaProviderVersion = JSS_MAJOR_VERSION + "." + 
+        String mozillaProviderVersion = JSS_MAJOR_VERSION + "." +
                                         JSS_MINOR_VERSION;
         if ( JSS_PATCH_VERSION != 0 ) {
             mozillaProviderVersion = mozillaProviderVersion + "." +
