@@ -33,12 +33,12 @@ import java.security.cert.CertificateException;
 import java.security.cert.X509Certificate;
 import java.util.Vector;
 
-import org.mozilla.jss.netscape.security.util.Utils;
 import org.mozilla.jss.netscape.security.util.BigInt;
 import org.mozilla.jss.netscape.security.util.DerInputStream;
 import org.mozilla.jss.netscape.security.util.DerOutputStream;
 import org.mozilla.jss.netscape.security.util.DerValue;
 import org.mozilla.jss.netscape.security.util.ObjectIdentifier;
+import org.mozilla.jss.netscape.security.util.Utils;
 import org.mozilla.jss.netscape.security.x509.AlgorithmId;
 import org.mozilla.jss.netscape.security.x509.X500Name;
 import org.mozilla.jss.netscape.security.x509.X509CertImpl;
@@ -207,6 +207,15 @@ public class PKCS7 {
         this.signerInfos = signerInfos;
     }
 
+    /**
+     * Construct PKCS7 from an array of certificates.
+     *
+     * @param certs Array of certificates.
+     */
+    public PKCS7(X509Certificate[] certs) {
+        this(new AlgorithmId[0], new ContentInfo(new byte[0]), certs, new SignerInfo[0]);
+    }
+
     private void parseSignedData(DerValue val)
             throws ParsingException, IOException {
 
