@@ -23,7 +23,7 @@ public class PK11PrivKey extends org.mozilla.jss.pkcs11.PK11Key
     private PK11PrivKey() { }
 
     protected PK11PrivKey(byte[] pointer) {
-        Assert._assert(pointer!=null);
+        assert(pointer!=null);
         keyProxy = new PrivateKeyProxy(pointer);
     }
 
@@ -52,7 +52,7 @@ public class PK11PrivKey extends org.mozilla.jss.pkcs11.PK11Key
         } else if (kt == KeyType.DSA) {
             return PrivateKey.Type.DSA;
         } else {
-            Assert._assert(kt == KeyType.EC);
+            assert(kt == KeyType.EC);
             return PrivateKey.Type.EC;
 	}
     }
@@ -67,6 +67,11 @@ public class PK11PrivKey extends org.mozilla.jss.pkcs11.PK11Key
      */
     public native int getStrength();
 
+    /**
+     * Returns the corresponding public key from a private key instance.
+     */
+    public native PK11PubKey getPublicKey();
+
     /**
      * Imports a PrivateKeyInfo, storing it as a temporary PrivateKey
      * on the given token.
