@@ -6,7 +6,7 @@ import org.mozilla.jss.crypto.PrivateKey;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-class PK11RSAPrivateKey
+public class PK11RSAPrivateKey
     extends PK11PrivKey implements java.security.interfaces.RSAPrivateKey
 {
     public static Logger logger = LoggerFactory.getLogger(PK11RSAPrivateKey.class);
