@@ -12,42 +12,58 @@ import java.security.PublicKey;
 import java.security.SecureRandom;
 import java.security.SignatureException;
 import java.security.spec.AlgorithmParameterSpec;
+import java.security.spec.PSSParameterSpec;
+import java.security.SecureRandom;
 
-import org.mozilla.jss.crypto.Algorithm;
-import org.mozilla.jss.crypto.NoSuchItemOnTokenException;
-import org.mozilla.jss.crypto.PrivateKey;
-import org.mozilla.jss.crypto.SignatureAlgorithm;
-import org.mozilla.jss.crypto.TokenException;
-import org.mozilla.jss.util.Assert;
-import org.mozilla.jss.util.NativeProxy;
+import org.mozilla.jss.crypto.*;
+import org.mozilla.jss.util.*;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-final class PK11Signature extends org.mozilla.jss.crypto.SignatureSpi {
+public final class PK11Signature
+    extends org.mozilla.jss.crypto.SignatureSpi
+    implements java.lang.AutoCloseable
+{
 
-	public PK11Signature(PK11Token token, SignatureAlgorithm algorithm)
-		throws NoSuchAlgorithmException, TokenException
-	{
+    public PK11Signature(PK11Token token, SignatureAlgorithm algorithm)
+        throws NoSuchAlgorithmException, TokenException
+    {
         assert(token!=null && algorithm!=null);
 
         // Make sure this token supports this algorithm.  It's OK if
         // it only supports the signing part; the hashing can be done
         // on the internal module.
-		if( ! token.doesAlgorithm(algorithm)  &&
+        if( ! token.doesAlgorithm(algorithm)  &&
             ! token.doesAlgorithm(algorithm.getSigningAlg()) )
         {
-			throw new NoSuchAlgorithmException();
-		}
+            throw new NoSuchAlgorithmException();
+        }
 
-		this.tokenProxy = token.getProxy();
-		this.token = token;
-		this.algorithm = algorithm;
-        if( algorithm.getRawAlg() == algorithm ) {
+        this.tokenProxy = token.getProxy();
+        this.token = token;
+        this.algorithm = algorithm;
+        this.digestAlgorithm = null;
+
+        /*
+         * A RSAPSSSignature would appear like "raw", because the algorithm
+         * matches the raw algorithm type, but we need additional parameters.
+         */
+        if (!isRSAPSSAlgorithm(algorithm) && algorithm.getRawAlg() == algorithm) {
             raw = true;
             rawInput = new ByteArrayOutputStream();
         }
-		this.state = UNINITIALIZED;
-	}
+        this.state = UNINITIALIZED;
+
+        // If we are using RSA-PSS, save the digest algorithm to be used by
+        // the native code.
+        if (isRSAPSSAlgorithm(algorithm)) {
+            try {
+                digestAlgorithm = algorithm.getDigestAlg();
+            } catch (NoSuchAlgorithmException e) {
+                digestAlgorithm = null;
+            }
+        }
+    }
 
 	public void engineInitSign(org.mozilla.jss.crypto.PrivateKey privateKey)
 		throws InvalidKeyException, TokenException
@@ -321,22 +337,96 @@ final class PK11Signature extends org.mozilla.jss.crypto.SignatureSpi {
     public void engineSetParameter(AlgorithmParameterSpec params)
         throws InvalidAlgorithmParameterException, TokenException
     {
-        throw new RuntimeException("PK11Signature.engineSetParameter() is not yet implemented");
+        // For now we only care about RSA PSS parameter specs
+        if (!isRSAPSSAlgorithm((SignatureAlgorithm) algorithm)) {
+            String msg = "Passing algorithm parameters for this algorithm (";
+            msg += algorithm + ") is not supported: " + params.toString();
+            throw new InvalidAlgorithmParameterException(msg);
+        }
+        
+        if (!(params instanceof PSSParameterSpec)) {
+            String msg = "Unsupported algorithm parameter spec class for ";
+            msg += "RSA/PSS: " + params.getClass().getName() + " -- ";
+            msg += params.toString();
+            throw new InvalidAlgorithmParameterException(msg);
+        }
+
+        if (params == null) {
+            String msg = "Got an unexpected null parameter spec for RSA/PSS";
+            throw new InvalidAlgorithmParameterException(msg);
+        }
+
+        digestAlgorithm = getRSAPSSDigestAlgFromSpec((PSSParameterSpec) params);
     }
 
-	protected PK11Token token;
-	protected TokenProxy tokenProxy;
-	protected Algorithm algorithm;
-	protected PK11Key key;
-	protected int state;
+    private Algorithm getRSAPSSDigestAlgFromSpec(PSSParameterSpec params)
+        throws InvalidAlgorithmParameterException
+    {
+        String hashAlgName = params.getDigestAlgorithm();
+        Algorithm hashAlg = null;
+
+        if (hashAlgName.equals("SHA-256")) {
+            hashAlg = DigestAlgorithm.SHA256;
+        } else if (hashAlgName.equals("SHA-384")) {
+            hashAlg = DigestAlgorithm.SHA384;
+        } else if (hashAlgName.equals("SHA-512")) {
+            hashAlg = DigestAlgorithm.SHA512;
+        } else {
+            String msg = "This digest algorithm (" + hashAlgName + ") isn't ";
+            msg += "supported for this algorithm (" + algorithm + "): ";
+            msg += params.toString();
+            throw new InvalidAlgorithmParameterException(msg);
+        }
+
+        return hashAlg;
+    }
+
+    private boolean isRSAPSSAlgorithm(SignatureAlgorithm algorithm) {
+        if (algorithm == null) {
+            return false;
+        }
+
+        if (algorithm == SignatureAlgorithm.RSAPSSSignatureWithSHA256Digest
+            || algorithm == SignatureAlgorithm.RSAPSSSignatureWithSHA384Digest 
+            || algorithm == SignatureAlgorithm.RSAPSSSignatureWithSHA512Digest
+            || algorithm == SignatureAlgorithm.RSAPSSSignature) {
+
+            return true;
+        }
+
+        return false;
+    }
+
+    @Override
+    public void finalize() throws Throwable {
+        close();
+    }
+
+    @Override
+    public void close() throws Exception {
+        if (sigContext != null) {
+            try {
+                sigContext.close();
+            } finally {
+                sigContext = null;
+            }
+        }
+    }
+
+    protected PK11Token token;
+    protected TokenProxy tokenProxy;
+    protected Algorithm algorithm;
+    protected Algorithm digestAlgorithm;
+    protected PK11Key key;
+    protected int state;
     protected SigContextProxy sigContext;
     protected boolean raw=false; // raw signing only, no hashing
     protected ByteArrayOutputStream rawInput;
 
-	// states
-	static public final int UNINITIALIZED = 0;
-	static public final int SIGN = 1;
-	static public final int VERIFY = 2;
+    // states
+    static public final int UNINITIALIZED = 0;
+    static public final int SIGN = 1;
+    static public final int VERIFY = 2;
 }
 
 class SigContextProxy extends NativeProxy {
